###############################################################
# Copyright (c) 2007 Xilinx, Inc. All Rights Reserved.
# You may copy and modify these files for your own internal use solely with
# Xilinx programmable logic devices and  Xilinx EDK system or create IP
# modules solely for Xilinx programmable logic devices and Xilinx EDK system.
# No rights are granted to distribute any files unless they are distributed in
# Xilinx programmable logic devices. 
###############################################################
#uses "xillib.tcl"

proc generate {drv_handle} {
    xdefine_mbox_config_files $drv_handle "xparameters.h" "xmbox_g.c" "XMbox"  
    xdefine_canonical_xpars $drv_handle "xparameters.h" "Mbox" "DEVICE_ID" "C_SPLB0_BASEADDR" "C_SPLB0_HIGHADDR" "C_NUM_CHANNELS"
}

#
# Create configuration C/H files as required by Xilinx drivers
#
proc xdefine_mbox_config_files {drv_handle hfile_name cfile_name drv_string} {

    # Open include file
    set hfile_handle [xopen_include_file $hfile_name]
    set cfile_name [file join "src" $cfile_name] 
    file delete $cfile_name
    set cfile_handle [open $cfile_name w]

    xprint_generated_header $cfile_handle "Driver configuration"    
    puts $cfile_handle "#include \"xparameters.h\""
    puts $cfile_handle "#include \"[string tolower $drv_string].h\""
    puts $cfile_handle "\n/*"
    puts $cfile_handle "* The configuration table for devices"
    puts $cfile_handle "*/\n"
    puts $cfile_handle [format "%s_Config %s_ConfigTable\[\] =" $drv_string $drv_string]
    puts $cfile_handle "\{"

    set periphs [xget_sw_iplist_for_driver $drv_handle]     
    set start_comma ""


    # Get all peripherals connected to this driver
    set periphs [xget_periphs $drv_handle] 

    # Copy over the right set of files as src based on processor type
    set sw_proc_handle [xget_libgen_proc_handle]
    set hw_proc_handle [xget_handle $sw_proc_handle "IPINST"]
    set proctype [xget_value $hw_proc_handle "OPTION" "IPNAME"]
   
    # Print all parameters for all peripherals
    set device_id 0
    set periphs [xget_sw_iplist_for_driver $drv_handle]     
    foreach periph $periphs {
	puts $hfile_handle ""
	puts $hfile_handle "/* Definitions for peripheral [string toupper [xget_hw_name $periph]] */"

        set mhs_handle [xget_hw_parent_handle $periph]
        set periph_name [string toupper [xget_value $periph "NAME"]]
        
        #-----------------------------------------------------------------------------------        
        # Handle Interface 0
        #-----------------------------------------------------------------------------------        
        set mbox_baseaddr    0
        set mbox_use_fsl     0
        set mbox_send_fsl    0
        set mbox_recv_fsl    0
        set if0_isfsl        0
        set if0_isplb        0
        
        set if0_isfsl [xget_value $periph "PARAMETER" "C_INTERFACE_0_IS_FSL"]
        if { $if0_isfsl == 1 && $proctype == "microblaze" } {
            # Make sure that the processor has the required instruction support
            if {[xget_value $hw_proc_handle "PARAMETER" "C_USE_EXTENDED_FSL_INSTR"] != 1 } {
                error "The mailbox driver requires parameter C_USE_EXTENDED_FSL_INSTR on MicroBlaze to be enabled when an FSL interface is used to connect the mailbox core." "" "mdt_error"                                
            }

            # Verify if this interface is connected to this processor
            # and set the send/receive FSL IDs
            # Query Recv FSL ID
            set drfsl0_bus [xget_hw_busif_value $periph "DRFSL0"]
            set drfsl0_busifs_list [xget_hw_connected_busifs_handle $mhs_handle $drfsl0_bus "TARGET"]            
            if { [llength $drfsl0_busifs_list] != 1 } {
                error "Unable to figure out FSL connectivity of Interface 0 on mailbox $periph_name." "" "mdt_error"                
            }

            set drfsl0_mbusif_name [xget_hw_name [lindex $drfsl0_busifs_list]]
            set mbif_found [regexp -nocase DRFSL\[0-9\]+ $drfsl0_mbusif_name] 
            if { $mbif_found == 0 } {
                error "DRFSL Interface 0 on mailbox $periph_name does not appear to be connected correctly to a MicroBlaze processor." "" "mdt_error"
            }
            scan $drfsl0_mbusif_name "DRFSL%d" recv_fsl 

            set dwfsl0_bus [xget_hw_busif_value $periph "DWFSL0"]
            set dwfsl0_busifs_list [xget_hw_connected_busifs_handle $mhs_handle $dwfsl0_bus "INITIATOR"]   
            if { [llength $dwfsl0_busifs_list] != 1 } {
                error "Unable to figure out FSL connectivity of Interface 0 on mailbox $periph_name." "" "mdt_error"                
            }

            set dwfsl0_mbusif_name [xget_hw_name [lindex $dwfsl0_busifs_list]]
            set mbif_found [regexp -nocase DWFSL\[0-9\]+ $dwfsl0_mbusif_name] 
            if { $mbif_found == 0 } {
                error "DWFSL Interface 0 on mailbox $periph_name does not appear to be connected correctly to a MicroBlaze processor." "" "mdt_error"
            }
            scan $dwfsl0_mbusif_name "DWFSL%d" send_fsl

            set mbox_baseaddr    0
            set mbox_use_fsl     1
            set mbox_send_fsl    $send_fsl
            set mbox_recv_fsl    $recv_fsl
            set if0_isfsl        1
            set if0_isplb        0
        } else {

            # Verify if this interface is connected to this processor
            ## Note:  CR 442125 If this is fixed the following code is an easier way to verify connectivity
            ## set master_ifs [xget_hw_connected_busifs_handle $mhs_handle "mb_plb_0" "MASTER"]
            #set plb [xget_hw_busif_value $periph "SPLB0"]           
            #foreach master_if $master_ifs {
            #    set master [xget_hw_parent_handle $master_if]
            #    if { $master == $hw_proc_handle } {
            #        set if0_connected 1
            #    }
            #}
            
            # Alternate scheme
            set plb [xget_hw_busif_value $periph "SPLB0"]           
            set plb_buses [xget_proc_dplb_buses $hw_proc_handle]
            foreach plb_bus $plb_buses {
                if { $plb_bus == $plb } {
                    set if0_isplb 1
                }
            }
            
            set mbox_baseaddr [xget_value $periph "PARAMETER" "C_SPLB0_BASEADDR"]
        }

        if { ($if0_isfsl == 1 && $proctype == "microblaze") || 
             $if0_isplb == 1 } {
            puts $hfile_handle [format "#define XPAR_%s_IF_0_DEVICE_ID %s" $periph_name $device_id]
            incr device_id
            puts $hfile_handle [format "#define XPAR_%s_IF_0_BASEADDR 0x%X" $periph_name $mbox_baseaddr]
            puts $hfile_handle [format "#define XPAR_%s_IF_0_USE_FSL %d" $periph_name $mbox_use_fsl]
            puts $hfile_handle [format "#define XPAR_%s_IF_0_SEND_FSL %d" $periph_name $mbox_send_fsl]
            puts $hfile_handle [format "#define XPAR_%s_IF_0_RECV_FSL %d" $periph_name $mbox_recv_fsl]            

            puts $cfile_handle [format "%s\t\{" $start_comma]
            puts $cfile_handle [format "\t\t XPAR_%s_IF_0_DEVICE_ID, " $periph_name]
            puts $cfile_handle [format "\t\t XPAR_%s_IF_0_BASEADDR, " $periph_name]            
            puts $cfile_handle [format "\t\t XPAR_%s_IF_0_USE_FSL," $periph_name]
            puts $cfile_handle [format "\t\t XPAR_%s_IF_0_SEND_FSL," $periph_name]
            puts $cfile_handle [format "\t\t XPAR_%s_IF_0_RECV_FSL" $periph_name]            
            puts -nonewline $cfile_handle "\n\t\}"
            set start_comma ",\n"
        }

        #-----------------------------------------------------------------------------------
        # Handle Interface 1
        #-----------------------------------------------------------------------------------        
        set mbox_baseaddr    0
        set mbox_use_fsl     0
        set mbox_send_fsl    0
        set mbox_recv_fsl    0
        set if1_isfsl        0
        set if1_isplb        0
        
        set if1_isfsl [xget_value $periph "PARAMETER" "C_INTERFACE_1_IS_FSL"]
        if { $if1_isfsl == 1 && $proctype == "microblaze" } {
            # Make sure that the processor has the required instruction support
            if {[xget_value $hw_proc_handle "PARAMETER" "C_USE_EXTENDED_FSL_INSTR"] != 1 } {
                error "The mailbox driver requires parameter C_USE_EXTENDED_FSL_INSTR on MicroBlaze to be enabled when an FSL interface is used to connect the mailbox core." "" "mdt_error"                                
            }

            # Verify if this interface is connected to this processor
            # and set the send/receive FSL IDs

            # Query Recv FSL ID
            set drfsl1_bus [xget_hw_busif_value $periph "DRFSL1"]
            set drfsl1_busifs_list [xget_hw_connected_busifs_handle $mhs_handle $drfsl1_bus "TARGET"]            
            if { [llength $drfsl1_busifs_list] != 1 } {
                error "Unable to figure out FSL connectivity of Interface 1 on mailbox $periph_name." "" "mdt_error"                
            }

            set drfsl1_mbusif_name [xget_hw_name [lindex $drfsl1_busifs_list]]
            set mbif_found [regexp -nocase DRFSL\[0-9\]+ $drfsl1_mbusif_name] 
            if { $mbif_found == 0 } {
                error "DRFSL Interface 1 on mailbox $periph_name does not appear to be connected correctly to a MicroBlaze processor." "" "mdt_error"
            }
            scan $drfsl1_mbusif_name "DRFSL%d" recv_fsl 

            set dwfsl1_bus [xget_hw_busif_value $periph "DWFSL1"]
            set dwfsl1_busifs_list [xget_hw_connected_busifs_handle $mhs_handle $dwfsl1_bus "INITIATOR"]   
            if { [llength $dwfsl1_busifs_list] != 1 } {
                error "Unable to figure out FSL connectivity of Interface 1 on mailbox $periph_name." "" "mdt_error"                
            }

            set dwfsl1_mbusif_name [xget_hw_name [lindex $dwfsl1_busifs_list]]
            set mbif_found [regexp -nocase DWFSL\[0-9\]+ $dwfsl1_mbusif_name] 
            if { $mbif_found == 0 } {
                error "DWFSL Interface 1 on mailbox $periph_name does not appear to be connected correctly to a MicroBlaze processor." "" "mdt_error"
            }
            scan $dwfsl1_mbusif_name "DWFSL%d" send_fsl

            set mbox_baseaddr    0
            set mbox_use_fsl     1
            set mbox_send_fsl    $send_fsl
            set mbox_recv_fsl    $recv_fsl
            set if0_isfsl        1
            set if0_isplb        0

        } else {

            # Verify if this interface is connected to this processor
            ## Note:  CR 442125 If this is fixed the following code is an easier way to verify connectivity
            ## set master_ifs [xget_hw_connected_busifs_handle $mhs_handle "mb_plb_0" "MASTER"]
            #set plb [xget_hw_busif_value $periph "SPLB0"]           
            #foreach master_if $master_ifs {
            #    set master [xget_hw_parent_handle $master_if]
            #    if { $master == $hw_proc_handle } {
            #        set if1_connected 1
            #    }
            #}
            
            # Alternate scheme
            set plb [xget_hw_busif_value $periph "SPLB1"]           
            set plb_buses [xget_proc_dplb_buses $hw_proc_handle]
            foreach plb_bus $plb_buses {
                if { $plb_bus == $plb } {
                    set if1_isplb 1
                }
            }
            
            set mbox_baseaddr [xget_value $periph "PARAMETER" "C_SPLB0_BASEADDR"]
        }

        if { ($if1_isfsl == 1 && $proctype == "microblaze") || 
             $if1_isplb == 1 } {
            puts $hfile_handle [format "#define XPAR_%s_IF_1_DEVICE_ID %s" $periph_name $device_id]
            incr device_id
            puts $hfile_handle [format "#define XPAR_%s_IF_1_BASEADDR 0x%X" $periph_name $mbox_baseaddr]
            puts $hfile_handle [format "#define XPAR_%s_IF_1_USE_FSL %d" $periph_name $mbox_use_fsl]
            puts $hfile_handle [format "#define XPAR_%s_IF_1_SEND_FSL %d" $periph_name $mbox_send_fsl]
            puts $hfile_handle [format "#define XPAR_%s_IF_1_RECV_FSL %d" $periph_name $mbox_recv_fsl]            

            puts $cfile_handle [format "%s\t\{" $start_comma]
            puts $cfile_handle [format "\t\t XPAR_%s_IF_1_DEVICE_ID, " $periph_name]
            puts $cfile_handle [format "\t\t XPAR_%s_IF_1_BASEADDR, " $periph_name]            
            puts $cfile_handle [format "\t\t XPAR_%s_IF_1_USE_FSL," $periph_name]
            puts $cfile_handle [format "\t\t XPAR_%s_IF_1_SEND_FSL," $periph_name]
            puts $cfile_handle [format "\t\t XPAR_%s_IF_1_RECV_FSL" $periph_name]            
            puts -nonewline $cfile_handle "\n\t\}"
            set start_comma ",\n"
        }
    }

    puts $cfile_handle "\n\};"
    puts $cfile_handle "\n";
    close $cfile_handle

    puts $hfile_handle [format "#define XPAR_XMBOX_NUM_INSTANCES %d" $device_id]
    puts $hfile_handle "\n/******************************************************************/\n"
    close $hfile_handle
}

## FIXME - This should go away when CR 442125 is fixed
proc xget_proc_dplb_buses { hw_proc_handle } {
    
    set proctype [xget_value $hw_proc_handle "OPTION" "IPNAME"]
    set dplb_buses [list]
    switch $proctype {
	"microblaze" { 
            lappend dplb_buses [xget_hw_busif_value $hw_proc_handle "DPLB"]
	}
        "ppc405_virtex4"  {
            lappend dplb_buses [xget_hw_busif_value $hw_proc_handle "DPLB0"]
            lappend dplb_buses [xget_hw_busif_value $hw_proc_handle "DPLB1"]            
        }
        "ppc440_virtex5"  {
            lappend dplb_buses [xget_hw_busif_value $hw_proc_handle "MPLB"]            
        }
        "default" {
            error "Unable to handle processor type $proctype" "" "mdt_error"
        }
    }

    return $dplb_buses
}
#
# Given a list of arguments, define each as a canonical constant name, using
# the driver name, in an include file.
#
proc xdefine_canonical_xpars {drv_handle file_name drv_string args} {
    # Open include file
    set file_handle [xopen_include_file $file_name]

    # Get all peripherals connected to this driver
    set periphs [xget_periphs $drv_handle]

    # Print canonical parameters for each peripheral
    set device_id 0
    foreach periph $periphs {
        puts $file_handle ""
        set periph_name [string toupper [xget_hw_name $periph]]
        set canonical_name [format "%s_%s" $drv_string $device_id]

        # Make sure canonical name is not the same as hardware instance
        if { [string compare -nocase $canonical_name $periph_name] == 0 } {
            # Abort canonical names
            close $file_handle
            return
        }

        puts $file_handle "/* Canonical definitions for peripheral $periph_name */"

        foreach arg $args {

            set lvalue [xget_dname $canonical_name $arg]

# The commented out rvalue is the name of the instance-specific constant
#           set rvalue [xget_name $periph $arg]

            # The rvalue set below is the actual value of the parameter
            set rvalue [xget_param_value $periph $arg]
            if {[llength $rvalue] == 0} {
                set rvalue 0
            }
            set rvalue [xformat_addr_string $rvalue $arg]

            puts $file_handle "#define $lvalue $rvalue"

        }
        incr device_id
        puts $file_handle ""
    }
    puts $file_handle "\n/******************************************************************/\n"
    close $file_handle
}

