/* $Id: xmbox_example.c,v 1.3 2007/07/20 17:13:04 meinelte Exp $ */
/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2007 Xilinx Inc.
*       All rights reserved.
*
*******************************************************************************/
/******************************************************************************/
/**
* @file mbox_example.c
*
* This file contains a design example for using the Mailbox hardware and driver
* XMbox
*
* The example assumes there are two processors availabile in the system that
* are expected to inter-communicate.
*
* This example has been tested on ML505 Hardware Evaluation board.
*
* @note
*
* These code fragments will illustrate how the XMbox component can be used to:
*   - Initialize the Mailbox core
*   - pass data between two processes
*
*
*<pre>
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1.00a va            First release
* 1.00a ecm  06/09/07 Cleanup, new coding standard, check into XCS
*</pre>
*******************************************************************************/

/***************************** Include Files **********************************/

#include "xmbox.h"
#include "xstatus.h"
#include "xparameters.h"

/************************** Constant Definitions *****************************/

#define MSGSIZ  1024

#define HELLO_SIZE 40

#if XPAR_CPU_ID == 2
#define MY_CPU_ID 1
#else
#define MY_CPU_ID XPAR_CPU_ID
#endif


/**************************** Type Definitions *******************************/

/***************** Macros (Inline Functions) Definitions *********************/

/************************** Variable Definitions *****************************/

char *role[2] = { "PRODUCER",
                  "CONSUMER" };

extern char data[];
char *sendmsg = data;
char recvmsg[MSGSIZ] __attribute__ ((aligned(4)));

u32 temp1pad = 0;
char *hello = "Hello! The Producer greets the Consumer...";
u32 temp2pad = 0;
char *rhello = "Hello! The Consumer greets the Producer...";

/************************** Function Prototypes ******************************/

int prodcon (void);

/******************************************************************************/
/**
*
* This function performs the producer and consumer console functionality.
* At compile time the role, producer or consumer, is determined
*
* @param    None
*
* @return   XST_SUCCESS
*
* @note     None
*
*******************************************************************************/
int prodcon ()
{

    XMbox mbox;
	XMbox_Config *ConfigPtr;
	XStatus status;
    u32 nbytes;
    u32 sent;
    u32  rcvd;
    int i;
    const int passes = 5;


    xil_printf ("(%s):\tStarts.\r\n", role[MY_CPU_ID]);


	/*
	 * Lookup configuration data in the device configuration table.
	 * Use this configuration info down below when initializing this component.
	 */

#ifdef XPAR_XPS_MAILBOX_0_IF_0_DEVICE_ID
#if XPAR_XPS_MAILBOX_0_IF_0_USE_FSL == 0
	ConfigPtr = XMbox_LookupConfig(XPAR_XPS_MAILBOX_0_IF_0_DEVICE_ID );
	if (ConfigPtr == (XMbox_Config *)XNULL){
		xil_printf ("(%s):\tLookupConfig Failed.%8.8x\r\n", role[MY_CPU_ID],XPAR_XPS_MAILBOX_0_IF_0_DEVICE_ID );
		return XST_FAILURE;
	}

#endif
#endif

#ifdef XPAR_XPS_MAILBOX_0_IF_1_DEVICE_ID
#if XPAR_XPS_MAILBOX_0_IF_1_USE_FSL == 0
	ConfigPtr = XMbox_LookupConfig(XPAR_XPS_MAILBOX_0_IF_1_DEVICE_ID );
	if (ConfigPtr == (XMbox_Config *)XNULL){
		xil_printf ("(%s):\tLookupConfig Failed.%8.8x\r\n", role[MY_CPU_ID],XPAR_XPS_MAILBOX_0_IF_1_DEVICE_ID );
		return XST_FAILURE;
	}

#endif
#endif

	/* This is the case for FSL, since the id's are the same foe both cores */

#ifdef XPAR_XPS_MAILBOX_0_IF_0_DEVICE_ID
#if XPAR_XPS_MAILBOX_0_IF_0_USE_FSL == 1
	ConfigPtr = XMbox_LookupConfig(XPAR_XPS_MAILBOX_0_IF_0_DEVICE_ID );
	if (ConfigPtr == (XMbox_Config *)XNULL){
		xil_printf ("(%s):\tLookupConfig Failed.%8.8x\r\n", role[MY_CPU_ID],XPAR_XPS_MAILBOX_0_IF_0_DEVICE_ID );
		return XST_FAILURE;
	}
#endif
#endif

	/*
	 * Perform the rest of the initialization
	 */

	status = XMbox_CfgInitialize(&mbox, ConfigPtr, ConfigPtr->BaseAddress);
    if (status != XST_SUCCESS){
		xil_printf ("(%s):\tXMbox_CfgInitialize Failed.\r\n", role[MY_CPU_ID]);
		return status;
	}
		xil_printf ("(DeviceId= %8.8x\r\n", ConfigPtr->DeviceId);
		xil_printf ("(BaseAddr= %8.8x\r\n", ConfigPtr->BaseAddress);
		xil_printf ("(UseFSL= %8.8x\r\n", ConfigPtr->UseFSL);
		xil_printf ("(SendID= %8.8x\r\n", ConfigPtr->SendID);
		xil_printf ("(RecvID= %8.8x\r\n", ConfigPtr->RecvID);

#ifdef ECM
#if MY_CPU_ID == 0
    /* First send the hello */
    nbytes = 0;
    xil_printf ("CPU 0 Write call\n");
    while (nbytes != HELLO_SIZE) {
        /* Write a message to the mbox */
        (void) XMbox_Write(&mbox, (u32*)((u8*)hello + nbytes), HELLO_SIZE - nbytes, &sent);

        nbytes += sent;
        xil_printf ("(%s):\tsent %d bytes.\r\n", role[MY_CPU_ID], sent);
    }

    xil_printf ("(%s):\tSuccessfully sent the message --> \r\n\r\n\t--[%s]--\r\n\r\n", role[MY_CPU_ID], hello);
#else
    /* First recv the hello */
    nbytes = 0;
    xil_printf ("CPU 1 Read call\n");
    while (nbytes < HELLO_SIZE) {
        /* Read a message from the mbox */
        (void)XMbox_Read(&mbox, (u32*)(recvmsg + nbytes), HELLO_SIZE - nbytes, &rcvd);

        nbytes += rcvd;
    	/*xil_printf ("nbytes = %8.8x, rcvd= %8.8x, Should be %8.8x\n", nbytes, rcvd, HELLO_SIZE);*/

    }

	xil_printf ("(%s):\tSuccessfully rcvd the message --> \r\n\r\n\t--[%s]--\r\n\r\n", role[MY_CPU_ID], recvmsg);
#endif
#endif


#if MY_CPU_ID == 0
    /* First send the hello */
    xil_printf ("CPU 0 WriteBlocking call\n");
	/* Write a message to the mbox */
	XMbox_WriteBlocking(&mbox, (u32*)((u8*)hello), HELLO_SIZE);

	xil_printf ("(%s):\tsent %d bytes.\r\n", role[MY_CPU_ID], HELLO_SIZE);

    xil_printf ("(%s):\tSuccessfully sent the message --> \r\n\r\n\t--[%s]--\r\n\r\n", role[MY_CPU_ID], hello);
#else
    /* First recv the hello */
    xil_printf ("CPU 1 ReadBlocking call\n");
	/* Read a message from the mbox */
	XMbox_ReadBlocking(&mbox, (u32*)(recvmsg),HELLO_SIZE);

    xil_printf ("(%s):\tSuccessfully rcvd the message --> \r\n\r\n\t--[%s]--\r\n\r\n", role[MY_CPU_ID], recvmsg);
#endif


#if MY_CPU_ID == 0
    /* Now get back a hello */
	/* Read a message from the mbox */
    xil_printf ("CPU 0 ReadBlocking call\n");
	XMbox_ReadBlocking (&mbox, (u32*)(recvmsg), HELLO_SIZE);

    xil_printf ("(%s):\tSuccessfully rcvd the message --> \r\n\r\n\t--[%s]--\r\n\r\n", role[MY_CPU_ID], recvmsg);
#else
    /* First recv the hello */
    nbytes = 0;
        /* Write a message to the mbox */
    xil_printf ("CPU 1 WriteBlocking call\n");
	XMbox_WriteBlocking (&mbox, (u32*)((u8*)rhello), HELLO_SIZE);

	xil_printf ("(%s):\tsent %d bytes.\r\n", role[MY_CPU_ID], sent);

    xil_printf ("(%s):\tSuccessfully sent the message --> \r\n\r\n\t--[%s]--\r\n\r\n", role[MY_CPU_ID], rhello);
#endif


#if MY_CPU_ID == 0
    /* Now send the full message */
    xil_printf ("(%s):\tNow sending the actual message -->\r\n", role[MY_CPU_ID]);
    /* Write a message to the mbox */
	XMbox_WriteBlocking (&mbox, (u32*)(sendmsg), MSGSIZ);


    xil_printf ("(%s):\tSuccessfully sent the full message.\r\n", role[MY_CPU_ID]);
#else
    /* Now recv the full message */
    xil_printf ("(%s):\tNow receiving the actual message -->\r\n", role[MY_CPU_ID]);
	/* Read a message from the mbox */
	 XMbox_ReadBlocking (&mbox, (u32*)(recvmsg), MSGSIZ);

    xil_printf ("(%s):\tSuccessfully rcvd the full message.\r\n", role[MY_CPU_ID]);

    if (memcmp (recvmsg, data, MSGSIZ)) {
        xil_printf ("(%s):\tError! Rcvd message does not match expected.\r\n", role[MY_CPU_ID]);
        return XST_FAILURE;
    }
    else {
        xil_printf ("(%s):\tSuccess! Rcvd message does match expected.\r\n", role[MY_CPU_ID]);
	}
#endif


    xil_printf ("(%s):\tEnds.\r\n", role[MY_CPU_ID]);
    return XST_SUCCESS;
}


int main ()
{
    xil_printf ("PRODCON :\tStarts.\r\n");

    if (prodcon () != XST_SUCCESS){
        xil_printf ("PRODCON :\tFailed.\r\n");
	}
    else {
        xil_printf ("PRODCON :\tSucceeded.\r\n");
	}

    xil_printf ("PRODCON :\tEnds.\r\n");

    return 0;
}
